@@ -28,7 +28,7 @@ use constant {
 use constant RE_UUID => qr/^[0-9a-fA-F]{8}-(?:[0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}$/;
 use constant RE_UINT => qr/^[1-9][0-9]*$/;
 
-our $VERSION = v0.03;
+our $VERSION = v0.04;
 
 my %attributes = (
     action      => {
@@ -52,21 +52,26 @@ my %attributes = (
         source_type => 'uint',
     },
     service => {},
+    best_service => {},
     (map {$_ => {source_type => 'number'}} qw(altitude latitude longitude)),
     space_object => {},
+
+    # TODO: define type.
+    (map {$_ => {source_type => 'string'}} qw(date_of_birth date_of_death)),
 );
 
 my %id_conv = (
     uuid => [qw(wikidata-identifier fellig-identifier oid)],
     oid  => [qw(uuid)],
     uri  => [qw(wikidata-identifier uuid oid)],
+    doi  => [qw(grove-art-online-identifier)],
 );
 
 my %lookup_services = (
-    uuid                            => [qw(Data::URIID)],
-    oid                             => [qw(Data::URIID)],
-    uri                             => [qw(Data::URIID)],
-    tagname                         => [], # none.
+    'uuid'                          => [qw(Data::URIID)],
+    'oid'                           => [qw(Data::URIID)],
+    'uri'                           => [qw(Data::URIID)],
+    'tagname'                       => [], # none.
     'wikidata-identifier'           => [qw(wikidata)],
     'musicbrainz-identifier'        => [qw(musicbrainz)],
     'british-museum-term'           => [qw(britishmuseum)],
@@ -74,12 +79,50 @@ my %lookup_services = (
     'fellig-box-number'             => [qw(fellig)],
     'fellig-identifier'             => [qw(fellig)],
     'youtube-video-identifier'      => [qw(youtube)],
-    e621tagtype                     => [qw(e621)],
+    'e621tagtype'                   => [qw(e621)],
     'wikimedia-commons-identifier'  => [qw(wikimedia-commons)],
     'e621-post-identifier'          => [qw(e621)],
     'xkcd-num'                      => [qw(xkcd)],
 );
 
+my %best_services = (
+    'wikidata-identifier'           => 'wikidata',
+    'musicbrainz-identifier'        => 'musicbrainz',
+    'british-museum-term'           => 'britishmuseum',
+    'gnd-identifier'                => 'dnb',
+    'fellig-box-number'             => 'fellig',
+    'fellig-identifier'             => 'fellig',
+    'youtube-video-identifier'      => 'youtube',
+    'e621tagtype'                   => 'e621',
+    'wikimedia-commons-identifier'  => 'wikimedia-commons',
+    'e621-post-identifier'          => 'e621',
+    'osm-node'                      => 'osm',
+    'osm-way'                       => 'osm',
+    'osm-relation'                  => 'osm',
+    'xkcd-num'                      => 'xkcd',
+    #'factgrid-identifier'           => '',
+    'viaf-identifier'               => 'viaf',
+    'open-library-identifier'       => 'open-library',
+    #'unesco-thesaurus-identifier'   => '',
+    #'isni'                          => '',
+    #'aev-identifier'                => '',
+    'europeana-entity-identifier'   => 'europeana',
+    'ngv-artist-identifier'         => 'ngv',
+    'ngv-artwork-identifier'        => 'ngv',
+    'geonames-identifier'           => 'geonames',
+    'find-a-grave-identifier'       => 'find-a-grave',
+    'libraries-australia-identifier'=> 'nla',
+    'nla-trove-people-identifier'   => 'nla',
+    'agsa-creator-identifier'       => 'agsa',
+    'amc-artist-identifier'         => 'amc',
+    'a-p-and-p-artist-identifier'   => 'a-p-and-p',
+    'tww-artist-identifier'         => 'tww',
+    'factgrid-identifier'           => 'factgrid',
+    'grove-art-online-identifier'   => 'grove-art-online',
+    'wikitree-person-identifier'    => 'wikitree-person',
+    'doi'                           => 'doi',
+);
+
 # Load extra services:
 {
     my $extra = Data::URIID::Service->_extra_lookup_services;
@@ -92,7 +135,7 @@ my %lookup_services = (
 }
 
 my %url_templates = (
-    wikidata => [
+    'wikidata' => [
         ['wikidata-identifier' => 'https://www.wikidata.org/wiki/%s'                    => qr/^Q/  => [qw(documentation info edit)]],
         ['wikidata-identifier' => 'https://www.wikidata.org/wiki/Property:%s'           => qr/^P/  => [qw(documentation info edit)]],
         ['wikidata-identifier' => 'https://www.wikidata.org/wiki/Special:EntityData/%s' => undef() => [qw(metadata)]],
@@ -100,59 +143,93 @@ my %url_templates = (
     'wikimedia-commons' => [
         ['wikimedia-commons-identifier' => 'https://commons.wikimedia.org/wiki/%s', undef, [qw(info render edit)]],
     ],
-    fellig => [
+    'fellig' => [
         map {
             [$_ => sprintf('https://www.fellig.org/subject/best/any/%s/%%s', $_), undef, [qw(info render)]],
             [$_ => sprintf('https://api.fellig.org/v0/overview/%s/%%s', $_), undef, [qw(metadata)]],
         } qw(fellig-identifier fellig-box-number uuid oid uri wikidata-identifier e621-post-identifier wikimedia-commons-identifier british-museum-term musicbrainz-identifier gnd-identifier e621tagtype)
     ],
-    youtube => [
+    'youtube' => [
         ['youtube-video-identifier' => 'https://www.youtube.com/watch?v=%s', undef, [qw(info render)]],
         ['youtube-video-identifier' => 'https://www.youtube.com/embed/%s',   undef, [qw(embed)]],
     ],
     'youtube-nocookie' => [
         ['youtube-video-identifier' => 'https://www.youtube-nocookie.com/embed/%s', undef, [qw(embed)]],
     ],
-    e621 => [
+    'e621' => [
         ['e621tagtype'          => 'https://e621.net/wiki_pages/show_or_new?title=%s', undef, [qw(info)]],
         ['e621-post-identifier' => 'https://e621.net/posts/%u',                        undef, [qw(render)]],
     ],
-    dnb => [
+    'dnb' => [
         ['gnd-identifier' => 'https://d-nb.info/gnd/%s', undef, [qw(info)]],
     ],
-    britishmuseum => [
+    'britishmuseum' => [
         ['british-museum-term' => 'https://www.britishmuseum.org/collection/term/%s', undef, [qw(info)]],
     ],
-    musicbrainz => [
+    'musicbrainz' => [
         ['musicbrainz-identifier' => 'https://musicbrainz.org/mbid/%s', undef, [qw(info)]],
         ['uuid'                   => 'https://musicbrainz.org/mbid/%s', undef, [qw(info)]],
     ],
-    osm => [
+    'osm' => [
         map {
             ['osm-'.$_ => sprintf('https://www.openstreetmap.org/%s/%%s', $_), undef, [qw(info render)]]
         } qw(node way relation)
     ],
-    xkcd => [
+    'xkcd' => [
         ['xkcd-num' => 'https://xkcd.com/%s/',              undef, [qw(info render)]],
         ['xkcd-num' => 'https://xkcd.com/%s/info.0.json',   undef, [qw(metadata)]],
     ],
-    viaf => [
+    'viaf' => [
         ['viaf-identifier' => 'https://viaf.org/viaf/%s/', undef, [qw(info)]],
     ],
-    europeana => [
+    'europeana' => [
         ['europeana-entity-identifier' => 'https://data.europeana.eu/%s', undef, [qw(info)], {no_escape => 1}],
     ],
     'open-library' => [
         ['open-library-identifier' => 'https://openlibrary.org/works/%s?mode=all', undef, [qw(info)]],
         ['open-library-identifier' => 'https://openlibrary.org/works/%s.json', undef, [qw(metadata)]],
     ],
-    ngv => [
+    'ngv' => [
         ['ngv-artist-identifier'  => 'https://www.ngv.vic.gov.au/explore/collection/artist/%s/', undef, [qw(info)]],
         ['ngv-artwork-identifier' => 'https://www.ngv.vic.gov.au/explore/collection/work/%s/',   undef, [qw(info)]],
     ],
-    geonames => [
+    'geonames' => [
         ['geonames-identifier' => 'https://www.geonames.org/%s', undef, [qw(info)]],
     ],
+    'find-a-grave' => [
+        ['find-a-grave-identifier' => 'https://www.findagrave.com/memorial/%s', undef, [qw(info)]],
+    ],
+    'nla' => [
+        ['libraries-australia-identifier' => 'https://nla.gov.au/anbd.aut-an%s', undef, [qw(info)]],
+        ['nla-trove-people-identifier'    => 'https://trove.nla.gov.au/people/%s', undef, [qw(info)]],
+    ],
+    'agsa' => [
+        ['agsa-creator-identifier' => 'https://www.agsa.sa.gov.au/collection-publications/collection/creators/_/%s/', undef, [qw(info)]],
+    ],
+    'amc' => [
+        ['amc-artist-identifier' => 'https://www.australianmusiccentre.com.au/artist/%s', undef, [qw(info)]],
+    ],
+    'a-p-and-p' => [
+        ['a-p-and-p-artist-identifier' => 'https://www.printsandprintmaking.gov.au/artists/%s/', undef, [qw(info)]],
+    ],
+    'tww' => [
+        ['tww-artist-identifier' => 'https://www.watercolourworld.org/artist/%s', undef, [qw(info)]],
+    ],
+    'factgrid' => [
+        ['factgrid-identifier' => 'https://database.factgrid.de/wiki/Item:%s'               => qr/^Q/  => [qw(documentation info edit)]],
+        ['factgrid-identifier' => 'https://database.factgrid.de/wiki/Property:%s'           => qr/^P/  => [qw(documentation info edit)]],
+        ['factgrid-identifier' => 'https://database.factgrid.de/wiki/Special:EntityData/%s' => undef() => [qw(metadata)]],
+    ],
+    'grove-art-online' => [
+        ['grove-art-online-identifier' => 'https://doi.org/10.1093/gao/9781884446054.article.%s', undef, [qw(info)]],
+    ],
+    'wikitree-person' => [
+        ['wikitree-person-identifier' => 'https://www.wikitree.com/wiki/%s', undef, [qw(info)]],
+    ],
+    'doi' => [
+        ['doi' => 'https://doi.org/%s',    undef, [qw(info)], {no_escape => 1}],
+        ['doi' => 'https://dx.doi.org/%s', undef, [qw(metadata)], {no_escape => 1}],
+    ],
 );
 
 my $re_yt_vid = qr#[^/]{11}#;
@@ -308,15 +385,114 @@ my %url_parser = (
             id => \1,
             action => 'info',
         },
+        {
+            host => qr#^(?:www,de,es,fr,fr-ca,it,nl,sv,pt)\.findagrave\.com$#,
+            path => qr#^/memorial/([1-9][0-9]*)$#,
+            source => 'find-a-grave',
+            type => 'find-a-grave-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'nla.gov.au',
+            path => qr#^/anbd\.aut-an([1-9][0-9]*)$#,
+            source => 'nla',
+            type => 'libraries-australia-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'trove.nla.gov.au',
+            path => qr#^/people/([1-9][0-9]*)$#,
+            source => 'nla',
+            type => 'nla-trove-people-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'www.agsa.sa.gov.au',
+            path => qr#^/collection-publications/collection/creators/_/([1-9][0-9]*)/$#,
+            source => 'agsa',
+            type => 'agsa-creator-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'www.australianmusiccentre.com.au',
+            path => qr#^/artist/([a-z]+(-[a-z]+)+)$#,
+            source => 'amc',
+            type => 'amc-artist-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'www.printsandprintmaking.gov.au',
+            path => qr#^/artists/([1-9][0-9]*)/$#,
+            source => 'a-p-and-p',
+            type => 'a-p-and-p-artist-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'www.watercolourworld.org',
+            path => qr#/artist/([\p{L}\d]+(-[\p{L}\d]+)*)$#,
+            source => 'tww',
+            type => 'tww-artist-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'database.factgrid.de',
+            path => qr#^/wiki/(?:Item|Property):([QP][1-9][0-9]*)$#,
+            source => 'factgrid',
+            type => 'factgrid-identifier',
+            id => \1,
+            ise_order => ISEORDER_RUO,
+            action => 'info',
+        },
+        {
+            host => 'database.factgrid.de',
+            path => qr#^/wiki/Special:EntityData/([QP][1-9][0-9]*)(?:\.[a-z]+)?$#,
+            source => 'factgrid',
+            type => 'factgrid-identifier',
+            id => \1,
+            ise_order => ISEORDER_RUO,
+            action => 'metadata',
+        },
+        {
+            host => 'www.wikitree.com',
+            path => qr#^/wiki/(\D+-[1-9][0-9]*)$#,
+            source => 'wikitree',
+            type => 'wikitree-person-identifier',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'doi.org',
+            path => qr#^/(10\.[0-9]{4,9}\/.+)$#,
+            source => 'doi',
+            type => 'doi',
+            id => \1,
+            action => 'info',
+        },
+        {
+            host => 'dx.doi.org',
+            path => qr#^/(10\.[0-9]{4,9}\/.+)$#,
+            source => 'doi',
+            type => 'doi',
+            id => \1,
+            action => 'metadata',
+        },
     ],
 );
 
 my %syntax = (
-    uuid                            => RE_UUID,
-    oid                             => qr/^[1-3](?:\.(?:0|[1-9][0-9]*))+$/,
-    uri                             => qr/^[a-zA-Z][a-zA-Z0-9\+\.\-]+:/,
-    tagname                         => qr/./,
+    'uuid'                          => RE_UUID,
+    'oid'                           => qr/^[1-3](?:\.(?:0|[1-9][0-9]*))+$/,
+    'uri'                           => qr/^[a-zA-Z][a-zA-Z0-9\+\.\-]+:/,
+    'tagname'                       => qr/./,
     'wikidata-identifier'           => qr/^[QP][1-9][0-9]*$/,
+    'factgrid-identifier'           => qr/^[QP][1-9][0-9]*$/,
     'wikimedia-commons-identifier'  => qr/^File:.*$/,
     'musicbrainz-identifier'        => RE_UUID,
     'british-museum-term'           => qr/^[A-Z]+[1-9][0-9]{0,5}$/, # TODO: Find good reference; See also: https://www.wikidata.org/wiki/Property:P1711#P1793
@@ -324,12 +500,14 @@ my %syntax = (
     'fellig-box-number'             => qr/^[1-9][0-9]{3}$/,
     'fellig-identifier'             => qr/^[A-Z]+[1-9][0-9]*$/,
     'youtube-video-identifier'      => qr/^.{11}$/,
-    e621tagtype                     => qr/./,
-    'e621-post-identifier'          => RE_UINT,
+    'e621tagtype'                   => qr/./,
+    'amc-artist-identifier'         => qr/^[a-z]+(-[a-z]+)+$/,
+    'tww-artist-identifier'         => qr/^[\p{L}\d]+(-[\p{L}\d]+)*$/,
+    'grove-art-online-identifier'   => qr/^T(?:0|20|22)\d{5}$/,
+    'wikitree-person-identifier'    => qr/^\D+-[1-9][0-9]*$/,
+    'doi'                           => qr/^10\.[0-9]{4,9}\/.+$/,
     (map {'osm-'.$_ => RE_UINT} qw(node way relation)),
-    'xkcd-num'                      => RE_UINT,
-    'ngv-artist-identifier'         => RE_UINT,
-    'ngv-artwork-identifier'        => RE_UINT,
+    (map {$_        => RE_UINT} qw(e621-post-identifier xkcd-num ngv-artist-identifier ngv-artwork-identifier find-a-grave-identifier libraries-australia-identifier nla-trove-people-identifier agsa-creator-identifier a-p-and-p-artist-identifier)),
 );
 
 my %fellig_tables = (
@@ -362,20 +540,29 @@ sub new {
 sub _set {
     my ($self, $service, $type, $id, $ise_order, $action) = @_;
     my Data::URIID $extractor = $self->extractor;
+    my $best_service;
+    my $type_name;
 
-    croak 'Invalid syntax for identifier type' unless $id =~ $syntax{$type};
+    $type      = $extractor->name_to_ise(type => $type);
+    $type_name = $extractor->ise_to_name(type => $type);
+
+    croak 'Invalid syntax for identifier type' unless $id =~ $syntax{$type_name};
 
     $ise_order //= ISEORDER_UOR;
     $service = $extractor->service($service) if defined $service;
-    $type    = $extractor->name_to_ise(type    => $type);
     $action  = $extractor->name_to_ise(action  => $action) if defined $action;
 
+    if (defined(my $best_service_name = $best_services{$type_name})) {
+        $best_service = $extractor->service($best_service_name);
+    }
+
     $self->{primary} = {
-        service     => $service,
-        type        => $type,
-        id          => $id,
-        ise_order   => $ise_order,
-        action      => $action,
+        service      => $service,
+        type         => $type,
+        id           => $id,
+        ise_order    => $ise_order,
+        action       => $action,
+        best_service => $best_service,
     };
     $self->{id} = {
         $type => $id,
@@ -398,7 +585,12 @@ sub _lookup {
         my %found = map {$_ => $rule->{$_}} qw(source type id ise_order action);
 
         if (defined $rule->{host}) {
-            next unless defined($host) && $host eq $rule->{host};
+            next unless defined $host;
+            if (ref($rule->{host})) {
+                next unless $host =~ $rule->{host};
+            } else {
+                next unless $host eq $rule->{host};
+            }
         }
         if (defined $rule->{path}) {
             next unless defined($path) && (@res = $path =~ $rule->{path});
@@ -572,7 +764,25 @@ sub ise {
     croak 'Identifier does not map to an ISE';
 }
 
+sub _as_lookup {
+    my ($self, $lookup_args, %opts) = @_;
+    my Data::URIID $extractor = $self->extractor;
+    my $res;
+    my $old_online;
 
+    if (exists $opts{online}) {
+        $old_online = $extractor->online;
+        $extractor->online($opts{online});
+    }
+
+    $res = $extractor->lookup(@{$lookup_args});
+
+    if (exists $opts{online}) {
+        $extractor->online($old_online);
+    }
+
+    return $res;
+}
 sub attribute {
     my ($self, $key, %opts) = @_;
     my $info = $attributes{$key} // croak sprintf('Unknown attribute "%s"', $key);
@@ -620,9 +830,9 @@ sub attribute {
             if ($as eq 'string' && eval {$value->isa('URI')}) {
                 return $cache->{$as} //= $value->as_string;
             } elsif ($as eq __PACKAGE__ && eval {$value->isa('URI')}) {
-                return $cache->{$as} //= $self->extractor->lookup($value);
+                return $cache->{$as} //= $self->_as_lookup([$value], %opts);
             } elsif ($as eq __PACKAGE__ && eval {$value->can('ise')}) {
-                return $cache->{$as} //= $self->extractor->lookup(ise => $value->ise);
+                return $cache->{$as} //= $self->_as_lookup([ise => $value->ise], %opts);
             } elsif ($as eq 'ise' && eval {$value->can('ise')}) {
                 return $cache->{$as} //= $value->ise;
             }
@@ -683,12 +893,14 @@ sub url {
         my $id_type = $extractor->ise_to_name(type => $self->id_type);
 
         foreach my $template (@{$url_templates{$service} // []}) {
+            my $t_opts = $template->[4] // {};
+
             next unless $id_type eq $template->[0];
 
             next if defined($template->[2]) && $id !~ $template->[2];
             next unless $self->_match_actions($template, \%opts);
 
-            return URI->new(sprintf($template->[1], uri_escape($id)));
+            return URI->new(sprintf($template->[1], $t_opts->{no_escape} ? $id : uri_escape_utf8($id)));
         }
     }
 
@@ -703,7 +915,7 @@ sub url {
         next if defined($t_id_filter) && $id !~ $t_id_filter;
         next unless $self->_match_actions($template, \%opts);
 
-        return URI->new(sprintf($t_template, $t_opts->{no_escape} ? $id : uri_escape($id)));
+        return URI->new(sprintf($t_template, $t_opts->{no_escape} ? $id : uri_escape_utf8($id)));
     }
 
     if (defined($opts{action}) && $opts{action} eq 'info' && $service eq 'wikipedia') {
@@ -761,6 +973,11 @@ sub _id_conv__uri__uuid {
 
 *_id_conv__uri__oid = *_id_conv__uri__uuid;
 
+sub _id_conv__doi__grove_art_online_identifier {
+    my ($self, $type_want, $type_name_have, $id) = @_;
+    $self->{id}{$type_want} = sprintf('10.1093/gao/9781884446054.article.%s', $id);
+}
+
 1;
 
 __END__
@@ -775,7 +992,7 @@ Data::URIID::Result - Extractor for identifiers from URIs
 
 =head1 VERSION
 
-version v0.03
+version v0.04
 
 =head1 SYNOPSIS
 
@@ -833,6 +1050,23 @@ The action the original URL was using.
 The altitude of the item.
 The reference system is not specified.
 
+=item C<best_service>
+
+The L<Data::URIID::Service> best used with this identifier.
+This is normally the service the identifier type originated from.
+
+=item C<date_of_birth>
+
+The date of birth of the person or object.
+
+B<Warning:> This is an experimental attribute and may be removed or changed later!
+
+=item C<date_of_death>
+
+The date of death of the person or object.
+
+B<Warning:> This is an experimental attribute and may be removed or changed later!
+
 =item C<description>
 
 A description of the subject.
@@ -899,6 +1133,11 @@ This can also be set to C<undef> to allow returning C<undef> in case of no value
 Overrides the default language tags from the C<$result-E<gt>extractor> object.
 May be an arrayref with a list of exact matches or a string that is parsed as a list (and supers being added).
 
+=item C<online>
+
+Overrides the L<Data::URIID/"online"> flag used for the lookup if C<as> is set to L<Data::URIID::Result>.
+This is very useful to prevent network traffic for auxiliary lookups.
+
 =back
 
 =head2 url
